# frozen_string_literal: true

require 'rails_helper'

RSpec.describe SpiritHub::TagsetSuggester, type: :service do
  let(:suggester) { described_class.new(tagset, user) }

  let(:tagset) { create(:tagset_doc) }
  let(:doc1) { create(:doc) }
  let(:doc2) { create(:doc) }
  let(:user) { create(:user) }

  before do
    allow(tagset).to receive(:docs).and_return([doc1, doc2])
  end

  describe '#suggest_spaces_to_classify' do
    subject { suggester.suggest_spaces_to_classify(limit: 3) }

    let(:doc1_spaces_suggestions) { [] }
    let(:doc2_spaces_suggestions) { [] }

    before do
      allow(doc1).to receive(:suggests_for).with('spaces', user:, limit: 3).and_return(doc1_spaces_suggestions)
      allow(doc2).to receive(:suggests_for).with('spaces', user:, limit: 3).and_return(doc2_spaces_suggestions)
    end

    it 'returns an empty array when nothing to suggest' do
      expect(subject).to eq []
    end

    context 'when there are space suggestions to the underlying docs' do
      let(:doc1_spaces_suggestions) { create_list(:tagset_directory, 1) }
      let(:doc2_spaces_suggestions) { create_list(:tagset_directory, 2) }

      it 'returns those as suggestions' do
        expect(subject).to match_array(doc1_spaces_suggestions + doc2_spaces_suggestions)
      end
    end

    context 'when there are space suggestions to the underlying docs, with common spaces' do
      let(:doc1_spaces_suggestions) { [create(:tagset_directory)] }
      let(:doc2_spaces_suggestions) { [doc1_spaces_suggestions.first, create(:tagset_directory)] }

      it 'returns those as suggestions, avoiding the duplicates' do
        expect(subject).to match_array [doc1_spaces_suggestions.first, doc2_spaces_suggestions.second]
      end
    end

    context 'when there are space suggestions to the underlying docs, above the limit' do
      let(:doc1_spaces_suggestions) { create_list(:tagset_directory, 2) }
      let(:doc2_spaces_suggestions) { create_list(:tagset_directory, 3) }

      it 'returns some of those as suggestions, within the limit' do
        spaces_suggestions = doc1_spaces_suggestions + doc2_spaces_suggestions

        expect((subject & spaces_suggestions).length).to eq 3
      end
    end
  end

  describe '#suggest_validation_comment_on' do
    subject { suggester.suggest_validation_comment_on(state) }

    let(:doc1_docs_suggestions) { [] }
    let(:doc2_docs_suggestions) { [] }

    before do
      allow(doc1).to receive(:suggests_for).with('docs', user:).and_return(doc1_docs_suggestions)
      allow(doc2).to receive(:suggests_for).with('docs', user:).and_return(doc2_docs_suggestions)
    end

    context 'when state "validate"' do
      let(:state) { 'validate' }

      it 'returns nil when nothing to suggest' do
        expect(subject).to be_nil
      end

      context 'when there are journal entries for similar and validated docs' do
        let(:doc1_docs_suggestions) { [validated_tagset1.reload.docs.first].compact }
        let(:doc2_docs_suggestions) { [validated_tagset2.reload.docs.first].compact }

        let(:validated_tagset1) { create(:tagset_doc).tap { |tagset| create(:doc, tagsets: [tagset]) } }
        let(:validated_tagset2) { create(:tagset_doc).tap { |tagset| create(:doc, tagsets: [tagset]) } }

        before do
          SpiritHub::JournalEntry.create!(code: 'DOC_VALIDATED', tagset_id: validated_tagset1.id, action_msg: 'All good')
          SpiritHub::JournalEntry.create!(code: 'DOC_VALIDATED', tagset_id: validated_tagset2.id, action_msg: 'Looks awesome')
        end

        it 'returns one of the msgs for the matched validated tagsets' do
          expect(doc1_docs_suggestions).to be_present
          expect(doc2_docs_suggestions).to be_present

          expect(subject).to be_in ['All good', 'Looks awesome']
        end
      end
    end

    context 'when state "refuse"' do
      let(:state) { 'refuse' }

      it 'returns nil when nothing to suggest' do
        expect(subject).to be_nil
      end

      context 'when there are journal entries for similar and validated docs' do
        let(:doc1_docs_suggestions) { [refused_tagset1.reload.docs.first].compact }
        let(:doc2_docs_suggestions) { [refused_tagset2.reload.docs.first].compact }

        let(:refused_tagset1) { create(:tagset_doc).tap { |tagset| create(:doc, tagsets: [tagset]) } }
        let(:refused_tagset2) { create(:tagset_doc).tap { |tagset| create(:doc, tagsets: [tagset]) } }

        before do
          SpiritHub::JournalEntry.create!(code: 'DOC_REFUSED', tagset_id: refused_tagset1.id, action_msg: 'Invalid data')
          SpiritHub::JournalEntry.create!(code: 'DOC_REFUSED', tagset_id: refused_tagset2.id, action_msg: 'Looks terrible')
        end

        it 'returns one of the msgs for the matched refused tagsets' do
          expect(doc1_docs_suggestions).to be_present
          expect(doc2_docs_suggestions).to be_present

          expect(subject).to be_in ['Invalid data', 'Looks terrible']
        end
      end
    end
  end
end
